@@ -106,7 +106,7 @@ namespace GtkSharp.Generation {
 			foreach (object member in members) {
 				if (member is Signal) {
 					Signal sig = member as Signal;
-					sw.WriteLine ("\t\t\tpublic IntPtr {0};", sig.ClassFieldName);
+					sw.WriteLine ("\t\t\tpublic IntPtr {0};", sig.CName.Replace ("\"", "").Replace ("-", "_"));
 				} else if (member is VirtualMethod) {
 					VirtualMethod vm = member as VirtualMethod;
 					bool has_target = methods [vm.Name] != null;
@@ -125,6 +125,7 @@ namespace GtkSharp.Generation {
 		{
 			sw.WriteLine ("\t\tstatic " + Name + "Adapter ()");
 			sw.WriteLine ("\t\t{");
+			sw.WriteLine ("\t\t\tGLib.GType.Register (_gtype, typeof({0}Adapter));", Name);
 			foreach (VirtualMethod vm in vms) {
 				bool has_target = methods [vm.Name] != null;
 				if (has_target && vm.IsValid)
@@ -169,6 +170,8 @@ namespace GtkSharp.Generation {
 			sw.WriteLine ();
 			sw.WriteLine ("\t\tpublic {0}Adapter ({0}Implementor implementor)", Name);
 			sw.WriteLine ("\t\t{");
+			sw.WriteLine ("\t\t\tif (implementor == null)");
+			sw.WriteLine ("\t\t\t\tthrow new ArgumentNullException (\"implementor\");");
 			sw.WriteLine ("\t\t\tthis.implementor = implementor;");
 			sw.WriteLine ("\t\t}");
 			sw.WriteLine ();
@@ -183,9 +186,11 @@ namespace GtkSharp.Generation {
 		{
 			Method m = GetMethod ("GetType");
 			m.GenerateImport (sw);
+			sw.WriteLine ("\t\tprivate static GLib.GType _gtype = new GLib.GType ({0} ());", m.CName);
+			sw.WriteLine ();
 			sw.WriteLine ("\t\tpublic override GLib.GType GType {");
 			sw.WriteLine ("\t\t\tget {");
-			sw.WriteLine ("\t\t\t\treturn new GLib.GType (" + m.CName +  " ());");
+			sw.WriteLine ("\t\t\t\treturn _gtype;");
 			sw.WriteLine ("\t\t\t}");
 			sw.WriteLine ("\t\t}");
 			sw.WriteLine ();
@@ -209,6 +214,11 @@ namespace GtkSharp.Generation {
 			sw.WriteLine ("\t\tpublic static " + Name + " GetObject (IntPtr handle, bool owned)");
 			sw.WriteLine ("\t\t{");
 			sw.WriteLine ("\t\t\tGLib.Object obj = GLib.Object.GetObject (handle, owned);");
+			sw.WriteLine ("\t\t\treturn GetObject (obj);");
+			sw.WriteLine ("\t\t}");
+			sw.WriteLine ();
+			sw.WriteLine ("\t\tpublic static " + Name + " GetObject (GLib.Object obj)");
+			sw.WriteLine ("\t\t{");
 			sw.WriteLine ("\t\t\tif (obj == null)");
 			sw.WriteLine ("\t\t\t\treturn null;");
 			sw.WriteLine ("\t\t\telse if (obj is " + Name + "Implementor)");
@@ -259,10 +269,8 @@ namespace GtkSharp.Generation {
 
 			GenProperties (gen_info, null);
 
-			foreach (Signal sig in sigs.Values) {
-				sig.GenCallback (sw);
+			foreach (Signal sig in sigs.Values)
 				sig.GenEvent (sw, null, "GLib.Object.GetObject (Handle)");
-			}
 
 			Method temp = methods ["GetType"] as Method;
 			if (temp != null)
@@ -296,7 +304,8 @@ namespace GtkSharp.Generation {
 
 			sw.WriteLine ();
 			sw.WriteLine ("\t[GLib.GInterface (typeof (" + Name + "Adapter))]");
-			sw.WriteLine ("\tpublic interface " + Name + "Implementor : GLib.IWrapper {");
+			string access = IsInternal ? "internal" : "public";
+			sw.WriteLine ("\t" + access + " interface " + Name + "Implementor : GLib.IWrapper {");
 			sw.WriteLine ();
 			Hashtable vm_table = new Hashtable ();
 			foreach (VirtualMethod vm in vms)
@@ -337,7 +346,8 @@ namespace GtkSharp.Generation {
 			sw.WriteLine ("\tusing System;");
 			sw.WriteLine ();
 			sw.WriteLine ("#region Autogenerated code");
-			sw.WriteLine ("\tpublic interface " + Name + " : GLib.IWrapper {");
+			string access = IsInternal ? "internal" : "public";
+			sw.WriteLine ("\t" + access + " interface " + Name + " : GLib.IWrapper {");
 			sw.WriteLine ();
 			
 			foreach (Signal sig in sigs.Values) {
